package RCWS;

import java.util.Locale;

import data.Input;
import map.MapTools;
import models.Node;
import models.SubCustomer;

/**
 * @author Angel A. Juan - http://ajuanp.wordpress.com
 * @version 130119
 */
public class Edge implements Comparable<Edge>
{
    /* INSTANCE FIELDS & CONSTRUCTOR */
    
    private Node origin; // origin node
    private Node end; // end node
    private double length = 0.0; // edge length
    private double savings = 0.0; // edge savings (Clarke & Wright)
    private Edge inverseEdge = null; // edge with inverse direction
    private Route inRoute = null; // evolving route containing this edge
	private float timeLength = 0f; // time needed to transit the edge
            
    public Edge(Node originNode, Node endNode) {   
        origin = originNode;
        end = endNode;
    }

    /* SET METHODS */
    
    public void setOrigin(Node origin) {
		this.origin = origin;
	}

	public void setEnd(Node end) {
		this.end = end;
	}

	public void setInverse(Edge e){inverseEdge = e;}
    public void setInRoute(Route r){inRoute = r;}
    public void setLength(double c){length = c;}
    public void setSavings(double s){savings = s;}
	public void setTimeLength(float t) {
		this.timeLength = t;	
	}

    /* GET METHODS */
    
    public Node getOrigin(){return origin;}
    public Node getEnd(){return end;}
    public double getLength(){return length;}
    public double getSavings(){return savings;}
    public Edge getInverseEdge(){return inverseEdge;}
    public Route getInRoute(){return inRoute;}
	public float getTimeLength() {return timeLength ;}

    /* AUXILIARY METHODS */
    
    /** Calculates edge costs  */
    public double calcLength(){   

        return MapTools.distanceMatrix[origin.getId()][end.getId()];
    }
    
    /** Calculates edge timeLength  */
    public float calcTimeLength(){   

    	float timeLength = MapTools.timeMatrix[origin.getId()][end.getId()];
    	
		Node iNode = this.getOrigin();
		Node jNode = this.getEnd();
		
		boolean sameCustomer = false;
		if (jNode instanceof SubCustomer && iNode instanceof SubCustomer) 
			sameCustomer = ((SubCustomer)jNode).getCustomer() == ((SubCustomer)iNode).getCustomer();
		
		boolean sameLocName = iNode.getLocName().equals(jNode.getLocName());
		
		if (sameLocName && !sameCustomer)
			timeLength += Input.getIntraCityTime();
		
        return timeLength;
    }

    /** Calculates edge savings as described in Clarke&Wright's heuristic */
    public double calcSavings(Node depot){   

        // Costs of originNode to depot
        double odC = MapTools.distanceMatrix[origin.getId()][depot.getId()];
     
        // Costs of depot to endNode
        double deC = MapTools.distanceMatrix[depot.getId()][end.getId()];
     
        // Costs of originNode to endNode
        double oeC = MapTools.distanceMatrix[origin.getId()][end.getId()];
     
        //Return cost depot to savings
        return odC + deC - oeC;
    }
    
    /** Compares two edges according to their respective savings */
    @Override
    public int compareTo(Edge otherEdge) {   
        Edge other = otherEdge;
        double s1 = this.getSavings();
        double s2 = other.getSavings();
        if (s1==s2) return 0;
        
        // list sorted in decreasing order, from more to less savings
        int r = -1; // default: this edge goes after other edge
        if( s1 < s2 ) // if this savings are larger than other savings
            r = 1; // this edge goes before other edge
        return r; 
    }

	@Override
	public String toString() {
		
		StringBuilder builder = new StringBuilder();
		builder.append("[").append(origin).append("->")
				.append(end).append("(")
				.append(String.format(Locale.US, "%.1f", getLength())).append(")]");
		return builder.toString();
	}
}