package RCWS;

import java.util.Arrays;
import java.util.Comparator;

import algorithm.Algorithm;
import main.RNG;
import map.SubMap;
import models.Depot;
import models.VehType;

public class WhRoute extends Route {
	
	private int[] purchase; 

	public WhRoute(int workingProducts, VehType vht) {
		super(workingProducts, vht);
	}
	
	public WhRoute(Route r) {
		super(r);	
	}

	public int[] getPurchase() {
		return purchase;
	}

	public void setPurchase(int[] purchase) {
		this.purchase = purchase;
	}
	
	public float calcProdTransferTime(float flow, float opSetupTime) {
		// Assuming full capacity purchase
		return 2 * this.getVehType().getTotalCapacity() / flow + getEdges().size() * opSetupTime;
	}
	
	public WhRoute makeSimpleCopy() {
		return new WhRoute(this);
	}

	/**
	 * Sorts an array of products according their associated needs
	 * @param needs
	 * @return sorted array of products
	 */
	private Integer[] productsOrderedByNeeds(int[] needs) {
		// Sort needs 
		class ArrayIndexComparator implements Comparator<Integer> {
		    private final int[] needs;

		    public ArrayIndexComparator(int[] needs) {
		        this.needs = needs;
		    }
		    
		    public Integer[] createIndexArray() {
		        Integer[] products = new Integer[needs.length];
		        for (int i = 0; i < needs.length; i++)
		            products[i] = i; 
		        return products;
		    }

		    @Override
		    public int compare(Integer product1, Integer product2)  {
		        return (needs[product1]>needs[product2]) ? -1 : (needs[product1]<needs[product2]) ? 1 : 0; 
		    }
		}
		
		ArrayIndexComparator comparator = new ArrayIndexComparator(needs);
		Integer[] sortedProducts = comparator.createIndexArray();
		Arrays.sort(sortedProducts, comparator);
		return sortedProducts;
	}	
	
	/**
	 * Calculate purchases of Wh-Route
	 * @param smPendDemand 
	 * @param sm
	 */
	public void calcPurchase(int[] smPendDemand, SubMap sm) {
		Depot depot = (Depot) getEdges().getFirst().getOrigin();
		int diffProducts = depot.getDiffProducts();
		int needs[] = new int[diffProducts];
		
		for (int i=0; i<diffProducts; i++)
			needs[i] = smPendDemand[i] - sm.getRemainingDepotInventory()[i];
		
		int[] purchase = new int[diffProducts];
		int[] ld = getLoadDistribution();
		int nEmptyComps = 0;
		// Firstly, book used compartments purchase (full capacity)
		for (int comp=0; comp<ld.length; comp++) {
			int product = ld[comp];
			if (product >= 0) {
				int qty = getVehType().getCapacity(comp);
				purchase[product] += qty;
				needs[product] -= qty;
			} else 
				nEmptyComps++;
		}
		
		// Secondly, empty compartments allows to choose product (most needed)
		
		while (nEmptyComps >0) {
			// Search randomly for first empty compartment
			int comp = RNG.getNextIntUD(ld.length);
			while (ld[comp] >= 0) 
				comp = (comp+1) % ld.length;
			// Compartment is empty, choose most needed product
			int qty = getVehType().getCapacity(comp);
			int mostNeededProduct = productsOrderedByNeeds(needs)[0];
			purchase[mostNeededProduct] += qty; 
			needs[mostNeededProduct] -= qty;
			ld[comp] = mostNeededProduct;
			nEmptyComps--;
		}
		
		setPurchase(purchase);
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("\n\t\t\t\t\t\t\tWhRoute [")
				.append(super.toString2())
				.append("\n\t\t\t\t\t\t\t\tpurchase=").append(Arrays.toString(purchase))
				.append("\n\t\t\t\t\t\t\t]");
		return builder.toString();
	}
}
