package data;

import java.util.List;

import models.Depot;
import models.Customer;
import models.Vehicle;
import models.Warehouse;

public class Input {

	private List<Customer> customers; 
	private List<Depot> depots;
	private List<Warehouse> warehouses;
	private final int diffProduct;
	private int totalDemand;
	private int totalVehCapacity;
	private float[][] distances;
	private float[][] times;
	private boolean euclideanDistance;
	private float maxTime;
	private float speed;
	private static float flow;
	private static float opSetupTime;
	private static float intraCityTime;
	private static int numNodes;
	
	public Input(List<Customer> customers, List<Depot> depots,
			List<Warehouse> warehouses, int diffProducts, float[][] distances, 
			float[][] times, float maxTime, int distType, float speed, float flow, 
			float opSetupTime, float intraCityTime) {
	
		this.customers = customers;
		this.depots = depots;
		this.warehouses = warehouses;
		this.diffProduct = diffProducts;
		for (Customer n : customers) 
			for (int prod = 0; prod < diffProducts; prod++)
				totalDemand += n.getDemand(prod);
		for (Depot d : depots) 
			for (Vehicle v : d.getVehicles())
				totalVehCapacity += v.getTotalCapacity() * v.getNumVehicles();
		this.distances = distances;
		this.times = times;
		this.euclideanDistance = (distType == 0);
		this.maxTime = maxTime;
		this.speed = speed;
		Input.flow = flow;
		Input.opSetupTime = opSetupTime;
		Input.intraCityTime = intraCityTime;
		Input.numNodes = customers.size()+depots.size()+warehouses.size();
	} 
	
	public int getNumCustomers() {
		return customers.size();
	}
	
	public int getNumDepots() { 
		return depots.size();
	}
	
	public int getNumWarehouses() {
		return warehouses.size();
	}
	
	public static int getNumNodes() {
		return numNodes;
	}
	
	public List<Customer> getCustomers() {
		return customers;
	}

	public void setCustomers(List<Customer> customers) {
		this.customers = customers;
	}

	public List<Depot> getDepots() {
		return depots;
	}

	public void setDepots(List<Depot> depots) {
		this.depots = depots;
	}

	public List<Warehouse> getWarehouses() {
		return warehouses;
	}

	public void setWarehouses(List<Warehouse> warehouses) {
		this.warehouses = warehouses;
	}

	public int getDiffProduct() {
		return diffProduct;
	}

	public int getTotalDemand() {
		return totalDemand;
	}

	public int getTotalVehCapacity() {
		return totalVehCapacity;
	}

	public float[][] getDistances() {
		return distances;
	}

	public boolean isEuclideanDistance() {
		return euclideanDistance;
	}

	public float getMaxTime() {
		return maxTime;
	}

	public float getSpeed() {
		return speed;
	}

	
	public static float getFlow() {
		return flow;
	}

	public static float getOpSetupTime() {
		return opSetupTime;
	}

	public static float getIntraCityTime() {
		return intraCityTime;
	}

	public float[][] getTimes() {
		return times;
	}


}
