package data;

import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import RCWS.RouteGroup;
import models.VehType;

public class Solution {
	private static long nInstances = 0; // number of instances
	private long id = 0; // solution id
	
	private final List<SubSol> subSolutions;

	private double cost = 0;
	private float timeLength = 0;
	
	public Solution() {
		nInstances++;
		id = nInstances;
		this.subSolutions = new LinkedList<SubSol>();
	}

	public List<SubSol> getSubSolutions() {
		return subSolutions;
	}
	
	public void addSubSol(SubSol subSol) {
		subSolutions.add(subSol);
		cost += subSol.getCosts();
		// Update critical time Length
		float subSolTimeLength = 0f;
		for (VehType v : subSol.getSMap().getRouteClassif().keySet()) {
			for (RouteGroup rg : subSol.getSMap().getRouteClassif().get(v)) {
				if (rg.getEndTime() > subSolTimeLength)
					subSolTimeLength = rg.getEndTime();
			}
		}
		subSol.setTimeLength(subSolTimeLength);
		if (subSolTimeLength > timeLength)
			timeLength = subSolTimeLength;
	}

	public double getCost() {
		return cost;
	}

	public float getTimeLength() {
		return timeLength;
	}

	public void setTimeLength(float timeLength) {
		this.timeLength = timeLength;
	}

	public long getId() {
		return id;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Solution [id=").append(id)
				.append(",\n\tsubSolutions=").append(subSolutions)
				.append(",\n\tcost=").append(String.format(Locale.US, "%.2f",cost))
				.append(",\n\ttimeLength=").append(String.format(Locale.US, "%.2f",timeLength))
				.append("\n]");
		return builder.toString();
	}

}
