package main;

/***********************************************************************************
* Project ILS-MDVRP - RNG.java
* This class manages the random behavior of the program. 
* Date of last revision (YYMMDD): 120327
* (C) Angel A. Juan - ajuanp(@)gmail.com
***********************************************************************************/

import java.util.Random;

/***********************************************************************************
* Project ILS-MDVRP - RNG.java
* This class manages the random behavior of the program. 
* Date of last revision (YYMMDD): 120327
* (C) Angel A. Juan - ajuanp(@)gmail.com
***********************************************************************************/

public class RNG {
	
    /*******************************************************************************
     *  INSTANCE FIELDS 
     ******************************************************************************/
	
	public static Random _rng;
	
	/*******************************************************************************
     *  CLASS CONSTRUCTOR 
     ******************************************************************************/
	
	public RNG(int seed) {
			
		_rng = new Random(seed);
	}

    /*******************************************************************************
	 * PUBLIC METHOD getPositionsArrayGD
     * returns an array of integers from 0 to n-1 randomly sorted using geometrical
     * distribution
     ******************************************************************************/
    
    public static int[] getPositionsArrayGD(int n, double beta) {
    
        int[] posArray = new int[n];
        int[] auxArray = new int[n]; // pointers to edges in savingsList

        // Reset auxArray
        for( int i = 0; i < n; i++ )
            auxArray[i] = i;

        // Assign new random positions
        for( int i = 0; i < n; i++ ) {  
            int pos = getNextIntGD(n-i, beta);
            posArray[i] = auxArray[pos];
            for( int j = pos; j < (n - i) - 1; j++ )
                auxArray[j] = auxArray[j + 1];
        }
        return posArray;
    }

	/*******************************************************************************
	 * PUBLIC METHOD getNextIntGD
     * returns a random number from 0 to n-1 using geometric distribution
     ******************************************************************************/
	
    public static int getNextIntGD(int n, double beta) {   
    	int pos;
    	double d = _rng.nextDouble();
        pos = (int)(Math.log(d) / Math.log(1 - beta));
        pos = pos % n;
        return pos;
    }

    
    /*******************************************************************************
	 * PUBLIC METHOD getNextIntUD
     * returns a random number from 0 to n-1 using uniform distribution
     ******************************************************************************/
	
    public static int getNextIntUD(int n) {   
    	int number;
        number = (int) (_rng.nextDouble() * n);
        return number;
    }
    
    /*******************************************************************************
	 * PUBLIC METHOD getNextIntUD
     * returns a random number from 0 to 1 using uniform distribution
     ******************************************************************************/
	
    public static float getNextFloat() {   
    	return (float)_rng.nextDouble();
    }

	public static Random getrng() {
		return _rng;
	}

    
    
}
