package map;

import java.util.Iterator;
import java.util.List;

import models.Depot;
import models.Customer;
import models.Node;
import models.Warehouse;
import data.Input;

public class MapTools {
	
	public static float[][] distanceMatrix;
	public static float[][] timeMatrix;
	public static float speed;

	private static float[][] distances(Input input) {
		int _nDepots = input.getNumDepots();
		int _nCustomers = input.getNumCustomers();
		int _nWH = input.getNumWarehouses();
		int _nNodes = _nCustomers + _nDepots + _nWH;
		List<Customer> _nodeList = input.getCustomers();
		List<Depot> _depotList = input.getDepots();
		List<Warehouse> _WhList = input.getWarehouses();

		float[][] _distances = new float[_nNodes][_nNodes];

		Customer ni, nj;
		Depot d;
		Warehouse wh;
		float distance;

		for (int i = 0; i < _nCustomers; i++) {
			_distances[i][i] = 0;
			ni = _nodeList.get(i);
			for (int j = i + 1; j < _nCustomers; j++) {
				nj = _nodeList.get(j);
				distance = distanceNodeToNode(ni, nj);
				_distances[i][j] = distance;
				_distances[j][i] = distance;
			}
			for (int j = 0; j < _nDepots; j++) {
				d = _depotList.get(j);
				distance = distanceNodeToNode(d, ni);
				_distances[i][j + _nCustomers] = distance;
				_distances[j + _nCustomers][i] = distance;
			}
			for (int j = 0; j < _nWH; j++) {
				wh = _WhList.get(j);
				distance = distanceNodeToNode(wh, ni);
				_distances[i][j + _nCustomers + _nDepots] = distance;
				_distances[j + _nCustomers + _nDepots][i] = distance;
			}
		}
		for (int i = 0; i < _nDepots; i++) {
			d = _depotList.get(i);
			for (int j = 0; j < _nWH; j++) {
				wh = _WhList.get(j);
				distance = distanceNodeToNode(wh, d);
				_distances[i + _nCustomers][j + _nCustomers + _nDepots] = distance;
				_distances[j + _nCustomers + _nDepots][i + _nCustomers] = distance;
			}
		}

		return _distances;
	}

	public static Savings calcDepotSavings(Input input) {
		
		if (!input.isEuclideanDistance()) {
			distanceMatrix = input.getDistances();
			timeMatrix = input.getTimes();
		}
		else {
			speed = input.getSpeed();
			distanceMatrix = distances(input);
			int nNodes = Input.getNumNodes();
			timeMatrix = new float[nNodes][nNodes];
			for (int i=0; i<nNodes; i++)
				for (int j=0; j<nNodes; j++)
					timeMatrix[i][j] = (distanceMatrix[i][j] / speed);
		}
			
		
		final float[][] savings = savings(input);
		
		Savings svg = new Savings(distanceMatrix, savings);
		
		return svg;
	}

	public static float[][] savings(Input input) {
		int _nDepots = input.getNumDepots();
		int _nNodes = input.getNumCustomers();
		float[][] _savings = new float[_nDepots][_nNodes];
		for (Depot d : input.getDepots()) {
			for (Customer n : input.getCustomers()) {
				_savings[d.getDepotId()][n.getId()] = minDistance(input, n, d) 
						- distanceMatrix[n.getId()][d.getId()];
			}
		}
		return _savings;
	}

	private static float minDistance(Input input, Customer n, Depot d) {

		float otherDistance;
		float minDistance = 0;
		// find minimum distance to other depots
		for (Depot depot : input.getDepots()) {
			if (!(depot.getId() == d.getId())) {
				otherDistance = distanceMatrix[n.getId()][depot.getId()];
				if (otherDistance < minDistance || minDistance == 0)
					minDistance = otherDistance;
			}
		}
		return minDistance;
	}

	private static float distanceNodeToNode(Node n1, Node n2) {
		return (float) Math.sqrt((n1.getX() - n2.getX())
				* (n1.getX() - n2.getX()) + (n1.getY() - n2.getY())
				* (n1.getY() - n2.getY()));
	}
	
	public static String str(Iterable<?> container, int nTabs, int maxLineLength) {
		int tabLength = 3;
		StringBuilder builder = new StringBuilder();
		Iterator<?> it = container.iterator();
		StringBuilder lp = new StringBuilder("\n");
		for (int i=0; i<nTabs; i++)
			lp.append("\t");
		String linePrefix = lp.toString();
		int currentLength = nTabs * tabLength;
		if (it.hasNext()) {
			String s = it.next().toString();
			currentLength += s.length();
			builder.append(s);
			while (it.hasNext()) {
				s = it.next().toString();
				builder.append(",");
				if (++currentLength + s.length() > maxLineLength && maxLineLength > 0) {
					builder.append(linePrefix);
					currentLength = nTabs * tabLength;
				}
				builder.append(s);
				currentLength += s.length();
			}
		}
		return builder.toString();
	}	


}
