package models;

import java.util.Comparator;

import RCWS.Route;

public class Vehicle implements Comparable<Vehicle> {


	private VehType vehType;
	
	private final int numVehicles;

	public Vehicle( VehType vehType, int numVehicles ) {
		this.vehType = vehType;
		this.numVehicles = numVehicles;
	}
		
	public void assignTo(Depot depot)  {
		depot.addVehicle(this);
	}

	public int getNumVehicles() {
		return numVehicles;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Vehicle [vehType=").append(vehType).append("]");
		return builder.toString();
	}

	public int getNumCompartments() {		
		return this.vehType.getNumCompartments();
	}

	public int getCapacity(int i) {
		return this.vehType.getCapacity(i);
	}

	public VehType getVehType() {
		return vehType;
	}

	public int getTotalCapacity() {
		return this.vehType.getTotalCapacity();
	}

	public double getFixedCost() {
		return this.vehType.getFixedCost();
	}

	public double getVaribleCost() {
		return this.vehType.getVaribleCost();
	}
	
	public boolean isWhCapable() {
		return this.vehType.isWhCapable();
	}
	
	@Override
	public int compareTo(Vehicle o) {
		return this.vehType.compareTo(o.vehType);
	}
	
	public static final Comparator<Vehicle> LESS_TOTAL_CAPACITY = new Comparator<Vehicle>() {
		public int compare(Vehicle o1, Vehicle o2) {
			
			return o1.compareTo(o2);
		}
	};
	public static final Comparator<Vehicle> MORE_TOTAL_CAPACITY = new Comparator<Vehicle>() {
		public int compare(Vehicle o1, Vehicle o2) {
			
			return o2.compareTo(o1);
		}
	};
}
