package plot;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.image.BufferedImage;
import java.util.Random;
import java.util.concurrent.TimeUnit;

import javax.swing.JFrame;
import javax.swing.JPanel;

import main.RNG;
import models.Depot;
import models.Customer;
import models.Warehouse;
import RCWS.Edge;
import RCWS.Route;
import data.Input;
import data.Solution;
import data.SubSol;

public class InstancePlot1 extends JPanel {
	private final Solution basedSol;
	private final Input in;
	
	private BufferedImage paintImage = new BufferedImage(1000, 700, BufferedImage.TYPE_3BYTE_BGR);
	
	
	public InstancePlot1(Solution sol, Input input) {
		this.basedSol = sol;
		this.in = input;
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.drawImage(paintImage, 0, 0, null);
	}
	
	public void plot(){
		Graphics g = paintImage.createGraphics();
	
		int factor = 5;
		
		Font stringFont = new Font( "SansSerif", Font.PLAIN, 10 );
		g.setFont(stringFont);
		
		//	Black nodes
		for(Customer n: in.getCustomers()) {
			g.setColor(Color.BLACK);
			g.drawOval(10+n.getX() *factor,10+n.getY() * factor, 4, 4);
			
			g.setColor(Color.DARK_GRAY);
//			StringBuffer sb = new StringBuffer(Integer.toString(n.getDemand(0)));
//			
//			for (int i=1; i<in.getDiffProduct(); i++)
//				sb.append(",").append(n.getDemand(i));
			
//			g.drawString("(" + n.getX() + "," + n.getY() + ")", 10+n.getX() *factor, 10+n.getY() * factor);
			g.drawString(n.getId() + "", 10+n.getX() *factor, 10+n.getY() * factor);
//			g.drawString(n.getId() + ":(" + sb + ")", 10+n.getX() *factor, 10+n.getY() * factor);
		}
		
		//	Red Depots
		g.setColor(Color.RED);
		for(SubSol ss: basedSol.getSubSolutions()) {
			Depot depot = ss.getSMap().getDepot();
			g.fillOval(10+depot.getX() * factor,10+depot.getY() * factor, 6, 6);
//			g.drawString(String.format("%.4f",ss.getCosts()), 10+depot.getX() * factor,10+depot.getY() * factor);
			g.drawString(depot.getId() + "", 10+depot.getX() *factor, 10+depot.getY() * factor);
		}
		
		//	Warehouses
		g.setColor(Color.CYAN.darker());
		for(Warehouse wh: in.getWarehouses()) {
			g.drawRect(10+wh.getX() * factor,10+wh.getY() * factor, 6, 6);
			g.drawString(wh.getId() + "", 10+wh.getX() *factor, 10+wh.getY() * factor);
		}
		
		//	Connections
		Color b1 = new Color(130,200,150);
		Color r1 = new Color(233,95,20);
		Color[] colors = new Color[]{Color.GRAY,r1,Color.BLUE,Color.MAGENTA,Color.PINK.darker(),Color.ORANGE,b1};
		int c = 0;
		
		for(SubSol ss: basedSol.getSubSolutions()) {
			for(Route r: ss.getRoutes()) {
				g.setColor(colors[c]);
				for(Edge e: r.getEdges())
					g.drawLine(10 + e.getOrigin().getX()*factor, 10 + e.getOrigin().getY()*factor,
							10 + e.getEnd().getX()*factor, 10 + e.getEnd().getY()*factor);
					
				for(Edge e: r.getEdges()) 
					drawLineArrow(10 + e.getOrigin().getX()*factor, 10 + e.getOrigin().getY()*factor,
							10 + e.getEnd().getX()*factor, 10 + e.getEnd().getY()*factor, g);
				
				c++;
				if(c == colors.length)c = 0;
			}
		}
		
		
		g.dispose();
		
        // repaint panel with new modified paint
		repaint();setVisible(true);
		while (true) {
			try {
				TimeUnit.MILLISECONDS.sleep(1000);
			} catch (InterruptedException e) {}
			repaint();
		}
	}

	private void drawLineArrow(int x0, int y0, int x1, int y1, Graphics g) {
		Random rng = RNG.getrng();
		double rnd = 0.2 + 0.8 * rng.nextDouble(); // positions arrow
		double length = 8d;            // arrow edge length
		double a = Math.toRadians(170d); // arrow aperture
		
		// Arrow position
		int aX = (int) (x0 + Math.round((x1 - x0) * rnd));
		int aY = (int) (y0 + Math.round((y1 - y0) * rnd));
		
		Double a0 = Math.atan((float)(y1-y0)/(float)(x1-x0));
		if (x0 > x1) a0 += Math.PI;     // line angle
		
		for (int i=-1; i <= 1; i+=2) {  // for each arrow edge 
	
			Double a1 = a0 + i * a;                         // arrow edge angle
			
			double vX = Math.round(length * Math.cos(a1));  // arrow edge vector
			double vY = Math.round(length * Math.sin(a1));
			
			g.drawLine(aX, aY, aX + (int)vX, aY + (int)vY); // draw arrow edge
		
		}
	}


//	public void plot() {
//		JFrame jf = new JFrame("Gasoil Problem");
//		
//		
//		jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//		jf.add(this);
//		jf.pack();
//		jf.setSize(1000, 700);
//		jf.setBackground(Color.WHITE);
//		
//		jf.setVisible(true);
//		while (true) {
//			try {
//				TimeUnit.MILLISECONDS.sleep(1000);
//			} catch (InterruptedException e) {}
//			jf.repaint();
//		}
//		
//	}
}
